require "lib.moonloader"
require "lib.nativeUI.textures"
require "lib.nativeUI.common"

local audio = {
    open = loadAudioStream("moonloader/lib/nativeUI/resources/audio/open.mp3"),
    close = loadAudioStream("moonloader/lib/nativeUI/resources/audio/close.mp3"),
    move = loadAudioStream("moonloader/lib/nativeUI/resources/audio/move.mp3"),
    buy = loadAudioStream("moonloader/lib/nativeUI/resources/audio/buy.mp3"),
    locked = loadAudioStream("moonloader/lib/nativeUI/resources/audio/locked.mp3"),
}

local INT_MENU = {}

function INT_MENU:create()
    local menu = INT_MENU
    menu.titleTexture = assert(common.mad.load_png_texture("moonloader/lib/nativeUI/resources/shopUI/interaction.png"))
    menu.titleText = "Native UI"
    menu.titleColor = {255, 255, 255}
    menu.x = 29
    menu.y = 10
    menu.w = 432
    menu.visible = false

    menu.structure = {
        panelNumber = 0,
        currentPanel = 0
    }

    menu.onInit = nil
    menu.onUpdate = nil

    menu.keys = {}
    menu.keys.down = 0x28
    menu.keys.up = 0x26
    menu.keys.active = 0x0D
    menu.keys.left = 0x25
    menu.keys.right = 0x27
    menu.keys.back = 0x08

    menu.openKey = 0x4D

    return menu
end

function INT_MENU:event_onInit(fx) self.onInit = fx end
function INT_MENU:event_onUpdate(fx) self.onUpdate = fx end

function INT_MENU:setStartPosX(startPosX) self.x = startPosX end
function INT_MENU:getStartPosX() return self.x end
function INT_MENU:setStartPosY(startPosY) self.y = startPosY end
function INT_MENU:getStartPosY() return self.y end
function INT_MENU:getStartedPos() return {self.x, self.y} end
function INT_MENU:setStartPos(x, y) self.x, self.y = x, y end

function INT_MENU:setOpenKey(key) self.openKey = key end
function INT_MENU:getOpenKey() return self.openKey end

function INT_MENU:setTitleText(titleText) self.titleText = titleText end
function INT_MENU:getTitleText() return self.titleText end

function INT_MENU:setTitleTextColor(R, G, B) self.titleColor = {R, G, B} end
function INT_MENU:getTitleColor() return self.titleColor end

function INT_MENU:setTitleTexture(titleTexture) self.titleTexture = assert(common.mad.load_png_texture(string.format("moonloader/lib/nativeUI/resources/shopUI/%s.png", titleTexture))) end
function INT_MENU:getTitleTexture() return self.titleTexture end

function INT_MENU:removeTitleBlock() self.titleTexture = nil end
function INT_MENU:removeTitleText() self.titleText = nil end

function INT_MENU:setVisible(visible) 
    if visible then
        if self.onInit ~= nil then self.onInit() end
    else
        if self.structure.panelNumber > 0 then
            self.structure["panel"..tostring(self.structure.currentPanel)].currentID = 1
            self.structure["panel"..tostring(self.structure.currentPanel)].subID = 0
            self.structure["panel1"].currentID = 1
            self.structure["panel1"].subID = 0
            self.structure.currentPanel = 1
        end
    end
    self.visible = visible 
end
function INT_MENU:isVisible() return self.visible end
function INT_MENU:swapVisible()
    if not self.visible and self.onInit ~= nil then self.onInit() end
    self.visible = not self.visible 
    if self.structure.panelNumber > 0 and not self.visible then
        self.structure["panel"..tostring(self.structure.currentPanel)].currentID = 1
        self.structure["panel"..tostring(self.structure.currentPanel)].subID = 0
        self.structure["panel1"].currentID = 1
        self.structure["panel1"].subID = 0
        self.structure.currentPanel = 1
    end
end

function INT_MENU:addPanel(panelHeader)
    if self.structure.currentPanel == 0 then self.structure.currentPanel = 1 end
    self.structure.panelNumber = self.structure.panelNumber + 1
    self.structure["panel"..tostring(self.structure.panelNumber)] = {}
    self.structure["panel"..tostring(self.structure.panelNumber)].currentID = 0
    self.structure["panel"..tostring(self.structure.panelNumber)].maxID = 0
    self.structure["panel"..tostring(self.structure.panelNumber)].subID = 0
    self.structure["panel"..tostring(self.structure.panelNumber)].color = {255, 255, 255}
    self.structure["panel"..tostring(self.structure.panelNumber)].prevPanel = 0
    self.structure["panel"..tostring(self.structure.panelNumber)].text = (panelHeader or "panel"..tostring(self.structure.panelNumber))
end

function INT_MENU:panelSetTextColor(r, g, b, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))].color = {r, g, b} end
function INT_MENU:panelGetTextColor(panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))].color end

function INT_MENU:addRow(text, panelID)
    panelID = panelID or self.structure.panelNumber

    if self.structure["panel"..tostring(panelID)].currentID == 0 then self.structure["panel"..tostring(panelID)].currentID = 1 end
    self.structure["panel"..tostring(panelID)].maxID = self.structure["panel"..tostring(panelID)].maxID + 1
    self.structure["panel"..tostring(panelID)]["row"..tostring(self.structure["panel"..tostring(panelID)].maxID)] = {}
    self.structure["panel"..tostring(panelID)]["row"..tostring(self.structure["panel"..tostring(panelID)].maxID)].text = text or "Row "..tostring(self.structure["panel"..tostring(panelID)].maxID)
    self.structure["panel"..tostring(panelID)]["row"..tostring(self.structure["panel"..tostring(panelID)].maxID)].type = 0
    -- controllable :
    -- 0 -> uncontrollable
    -- 1 ->  user control
    -- -1 -> nativeUI control (if implemented)
    -- 2 -> user + nativeUI control
    self.structure["panel"..tostring(panelID)]["row"..tostring(self.structure["panel"..tostring(panelID)].maxID)].controllable = 2
    self.structure["panel"..tostring(panelID)]["row"..tostring(self.structure["panel"..tostring(panelID)].maxID)].locked = 0
    self.structure["panel"..tostring(panelID)]["row"..tostring(self.structure["panel"..tostring(panelID)].maxID)].new = false
    self.structure["panel"..tostring(panelID)]["row"..tostring(self.structure["panel"..tostring(panelID)].maxID)].enabled = true
    self.structure["panel"..tostring(panelID)]["row"..tostring(self.structure["panel"..tostring(panelID)].maxID)].unique = false
    self.structure["panel"..tostring(panelID)]["row"..tostring(self.structure["panel"..tostring(panelID)].maxID)].description = {nil, 0}
    self.structure["panel"..tostring(panelID)]["row"..tostring(self.structure["panel"..tostring(panelID)].maxID)].action = {nil, nil}
end

function INT_MENU:rowGetType(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring(panelID)].maxID))].type end
function INT_MENU:rowSetType(type, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring(panelID)].maxID))].type = type end
function INT_MENU:rowSetControlType(type, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].controllable = type end
function INT_MENU:rowGetControlType(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].controllable end
function INT_MENU:rowSetAction(fx, args, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].action = {fx, (args or {})} end
function INT_MENU:rowSetLocked(lockType, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].locked = lockType end
function INT_MENU:rowIsLocked(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring(panelID)].maxID))].locked end
function INT_MENU:rowSetNew(new, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].new = new end
function INT_MENU:rowIsNew(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring(panelID)].maxID))].new end
function INT_MENU:rowSetEnabled(enabled, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].enabled = enabled end
function INT_MENU:rowIsEnabled(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring(panelID)].maxID))].enabled end
function INT_MENU:rowSetDescription(description, lines, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].description = {description, (lines or 1)} end
function INT_MENU:rowGetDescription(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].description[1] end
function INT_MENU:rowGetDescriptionLines(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].description[2] end
function INT_MENU:rowGetText(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].text end
function INT_MENU:rowSetText(text, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].text = text end

function INT_MENU:rowSwitchTo_TextRow(rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type = 0 end
function INT_MENU:rowIsTextRow(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 0 end

function INT_MENU:rowAddProperty_Redirect(toPanel, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].toPanel = toPanel end
function INT_MENU:rowSwitchTo_RedirectRow(rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type = 1 end
function INT_MENU:redirectRow_getRedirectionPanelID(rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 1 then return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].toPanel end end
function INT_MENU:redirectRow_setRedirectionPanelID(toPanel, rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 1 then self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].toPanel = toPanel end end
function INT_MENU:rowIsRedirectRow(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 1 end

function INT_MENU:rowAddProperty_Checkbox(checked, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].checked = checked end
function INT_MENU:rowSwitchTo_CheckboxRow(rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type = 2 end
function INT_MENU:checkboxRow_Checked(checked, rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 2 then self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].checked = checked end end
function INT_MENU:checkboxRow_IsChecked(rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 2 then return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].checked end end
function INT_MENU:checkboxRow_SwapCheck(rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 2 then self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].checked = not self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].checked end end
function INT_MENU:rowIsCheckboxRow(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 2 end

function INT_MENU:rowAddProperty_Crossbox(crossed, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].crossed = crossed end
function INT_MENU:rowSwitchTo_CrossboxRow(rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type = 3 end
function INT_MENU:crossboxRow_Crossed(crossed, rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 3 then self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].crossed = crossed end end
function INT_MENU:crossboxRow_IsCrossed(rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 3 then return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].crossed end end
function INT_MENU:crossboxRow_SwapCross(rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 3 then self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].crossed = not self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].crossed end end
function INT_MENU:rowIsCrossboxRow(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 3 end

function INT_MENU:rowSwitchTo_TickRow(rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type = 4 end
function INT_MENU:rowIsTickRow(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 4 end

function INT_MENU:rowAddProperty_RightText(rtext, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].rtext = rtext end
function INT_MENU:rowSwitchTo_RightTextRow(rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type = 5 end
function INT_MENU:rightTextRow_setText(rtext, rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 5 then self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].rtext = rtext end end
function INT_MENU:rightTextRow_getText(rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 5 then return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].rtext end end
function INT_MENU:rowIsRightTextRow(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 5 end

function INT_MENU:rowAddProperty_Price(price, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].price = price end
function INT_MENU:rowSwitchTo_PriceRow(rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type = 6 end
function INT_MENU:rowSwitchTo_ChipsRow(rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type = 7 end
function INT_MENU:rowIsPriceRow(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 6 end
function INT_MENU:rowIsChipsRow(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 7 end
function INT_MENU:priceRow_setPrice(price, rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 6 then self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].price = price end end
function INT_MENU:chipsRow_setPrice(price, rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 7 then self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].price = price end end
function INT_MENU:priceRow_getPrice(rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 6 then return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].price end end
function INT_MENU:chipsRow_getPrice(rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 7 then return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].price end end

function INT_MENU:rowAddProperty_List(list, rowID, panelID) 
    self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].list = list
    self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].listIndex = 1
end
function INT_MENU:rowSwitchTo_ListRow(rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type = 8 end
function INT_MENU:listRow_getListIndex(rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 8 then return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].listIndex end end
function INT_MENU:rowIsListRow(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 8 end

-- 1 gold, 2 silver, 3 bronze
function INT_MENU:rowAddProperty_Medal(medal, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].medal = medal end
function INT_MENU:rowSwitchTo_MedalRow(rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type = 9 end
function INT_MENU:rowIsMedalRow(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 9 end

function INT_MENU:rowAddProperty_Badge(badge, rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].badge = {badge, false} end
function INT_MENU:rowSwitchTo_BadgeRow(rowID, panelID) self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type = 10 end
function INT_MENU:badgeRow_setVisible(visible, rowID, panelID) 
    if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 10 then 
        if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].unique then self:badgeRow_setVisibleToAll(false, (panelID or self.structure.panelNumber)) end
        self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].badge[2] = visible
    end
end
function INT_MENU:badgeRow_isVisible(rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 10 then return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].badge[2] end end
function INT_MENU:badgeRow_swapVisible(rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 10 then self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].badge[2] = not self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].badge[2] end end
function INT_MENU:badgeRow_setUnique(unique, rowID, panelID) if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 10 then self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].unique = unique end end
function INT_MENU:rowIsBadgeRow(rowID, panelID) return self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring((rowID or self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID))].type == 10 end

function INT_MENU:badgeRow_setVisibleToAll(visible, panelID)
    for i = 1, self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID do
        if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring(i)].type == 10 then
            self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring(i)].badge[2] = visible
        end
    end
end

function INT_MENU:badgeRow_setUniqueToAll(unique, panelID)
    for i = 1, self.structure["panel"..tostring((panelID or self.structure.panelNumber))].maxID do
        if self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring(i)].type == 10 then
            self.structure["panel"..tostring((panelID or self.structure.panelNumber))]["row"..tostring(i)].unique = unique
        end
    end
end

function INT_MENU:itemDown()
    if self.structure["panel"..tostring(self.structure.currentPanel)].currentID == 10 + self.structure["panel"..tostring(self.structure.currentPanel)].subID then self.structure["panel"..tostring(self.structure.currentPanel)].subID = self.structure["panel"..tostring(self.structure.currentPanel)].subID + 1 end
    self.structure["panel"..tostring(self.structure.currentPanel)].currentID = self.structure["panel"..tostring(self.structure.currentPanel)].currentID + 1
    if self.structure["panel"..tostring(self.structure.currentPanel)].currentID > self.structure["panel"..tostring(self.structure.currentPanel)].maxID then 
        self.structure["panel"..tostring(self.structure.currentPanel)].currentID = 1 
        self.structure["panel"..tostring(self.structure.currentPanel)].subID = 0 
    end
end

function INT_MENU:itemUp()
    if self.structure["panel"..tostring(self.structure.currentPanel)].currentID == self.structure["panel"..tostring(self.structure.currentPanel)].subID + 1 then self.structure["panel"..tostring(self.structure.currentPanel)].subID = self.structure["panel"..tostring(self.structure.currentPanel)].subID - 1 end
    self.structure["panel"..tostring(self.structure.currentPanel)].currentID = self.structure["panel"..tostring(self.structure.currentPanel)].currentID - 1
    if self.structure["panel"..tostring(self.structure.currentPanel)].currentID < 1 then 
        self.structure["panel"..tostring(self.structure.currentPanel)].currentID = self.structure["panel"..tostring(self.structure.currentPanel)].maxID 
        self.structure["panel"..tostring(self.structure.currentPanel)].subID = self.structure["panel"..tostring(self.structure.currentPanel)].maxID - 10
        if self.structure["panel"..tostring(self.structure.currentPanel)].subID < 0 then self.structure["panel"..tostring(self.structure.currentPanel)].subID = 0 end
    end
end

function INT_MENU:itemPrev()
    self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].listIndex = self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].listIndex - 1
    if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].listIndex < 1 then
        self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].listIndex = #self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].list
    end
end

function INT_MENU:itemNext()
    self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].listIndex = self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].listIndex + 1
    if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].listIndex > #self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].list then
        self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].listIndex = 1
    end
end

function INT_MENU:performEvent(event)
    if event == "open" then 
        if self.visible then setAudioStreamState(audio.close, 1) end
        self:swapVisible()
        setAudioStreamState(audio.open, 1)
    end

    if self.structure.panelNumber > 0 then
        if self.structure["panel"..tostring(self.structure.currentPanel)].maxID > 0 then
            rowType = self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].type
            controlType = self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].controllable
        end
        
        if event == "down" then 
            self:itemDown()
            setAudioStreamState(audio.move, 1)
        elseif event == "up" then 
            self:itemUp()
            setAudioStreamState(audio.move, 1)
        elseif event == "left" and rowType == 8 then 
            self:itemPrev()
            setAudioStreamState(audio.move, 1)
        elseif event == "right" and rowType == 8 then 
            self:itemNext()
            setAudioStreamState(audio.move, 1)
        elseif event == "back" then
            setAudioStreamState(audio.close, 1)
            if self.structure["panel"..tostring(self.structure.currentPanel)].prevPanel == 0 then
                self:swapVisible()
            else
                self.structure["panel"..tostring(self.structure.currentPanel)].currentID = 1
                self.structure["panel"..tostring(self.structure.currentPanel)].subID = 0
                self.structure.currentPanel = self.structure["panel"..tostring(self.structure.currentPanel)].prevPanel
            end
        elseif event == "active" then
            if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].enabled and self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].locked == 0 then
                if controlType >= 1 then
                    if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].action[1] ~= nil then
                        self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].action[1](unpack(self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].action[2])) 
                    end
                end
                if controlType == -1 or controlType == 2 then 
                    if rowType == 2 then 
                        setAudioStreamState(audio.move, 1)
                        self:checkboxRow_SwapCheck(self.structure["panel"..self.structure.currentPanel].currentID, self.structure.currentPanel)
                    elseif rowType == 3 then 
                        setAudioStreamState(audio.move, 1)
                        self:crossboxRow_SwapCross(self.structure["panel"..self.structure.currentPanel].currentID, self.structure.currentPanel)
                    elseif rowType == 6 then 
                        setAudioStreamState(audio.buy, 1)
                        givePlayerMoney(playerHandle, - self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].price)
                    elseif rowType == 10 then 
                        setAudioStreamState(audio.move, 1)
                        if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].unique then self:badgeRow_setVisibleToAll(false, self.structure.currentPanel) end
                        self:badgeRow_SwapVisible(self.structure["panel"..self.structure.currentPanel].currentID, self.structure.currentPanel)
                    elseif rowType == 1 then
                        setAudioStreamState(audio.open, 1)
                        self.structure["panel"..self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].toPanel].prevPanel = self.structure.currentPanel
                        self.structure.currentPanel = self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].toPanel
                    end
                end
            else
                setAudioStreamState(audio.locked, 1)
            end
        end
    end
end

function INT_MENU:show()

    if self.visible then
        clearHelp()

        if self.onUpdate ~= nil then self.onUpdate() end

        local y = self.y
        local lenght, color, prop

        -- title block

        if self.titleTexture ~= nil then
            common.drawTexture(self.titleTexture, self.x, y, self.x + self.w, y + 108)
            if self.titleText ~= nil then
                common.drawText(self.titleText, self.x + (self.w/2), y + 12, "GOTHIC", 1.8, 3.8, "CENTER", self.w, self.titleColor[1], self.titleColor[2], self.titleColor[3], 255, 1, 0)
            end
            y = y + 108
        end

        -- panel header

        if self.structure.panelNumber > 0 then
            common.drawTexture(textures.rect, self.x, y, self.x + self.w, y + 37, 0, 0, 0)
            common.drawText(self.structure["panel"..tostring(self.structure.currentPanel)].text, self.x + 5, y + 2, "MENU", 0.6, 1.7, "LEFT", self.w, self.structure["panel"..tostring(self.structure.currentPanel)].color[1], self.structure["panel"..tostring(self.structure.currentPanel)].color[2], self.structure["panel"..tostring(self.structure.currentPanel)].color[3], 255, 0, 0)
            if self.structure["panel"..tostring(self.structure.currentPanel)].maxID > 0 then
                common.drawText(string.format("%d/%d", self.structure["panel"..tostring(self.structure.currentPanel)].currentID, self.structure["panel"..tostring(self.structure.currentPanel)].maxID), self.x + self.w - 5, y + 2, "MENU", 0.6, 1.7, "RIGHT", - self.w, self.structure["panel"..tostring(self.structure.currentPanel)].color[1], self.structure["panel"..tostring(self.structure.currentPanel)].color[2], self.structure["panel"..tostring(self.structure.currentPanel)].color[3], 255, 0, 0)
            end
        
            y = y + 37

            -- panel body

            if self.structure["panel"..tostring(self.structure.currentPanel)].maxID > 10 then lenght = 10 else lenght = self.structure["panel"..tostring(self.structure.currentPanel)].maxID end
            
            common.drawTexture(textures.bgdTexture, self.x, y, self.x + self.w, y + (38 * lenght))

            for i = 1 + self.structure["panel"..tostring(self.structure.currentPanel)].subID, lenght + self.structure["panel"..tostring(self.structure.currentPanel)].subID do

                local rowType = self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].type
                
                if i == self.structure["panel"..tostring(self.structure.currentPanel)].currentID then 
                    common.drawTexture(textures.navTexture, self.x, y, self.x + self.w, y + 38)
                    color = {0, 0, 0}
                    prop = "b"
                    if rowType == 8 then prop = "< %s >" end
                else
                    color = {255, 255, 255}
                    prop = ""
                    if rowType == 8 then prop = "%s" end
                end
                
                if not self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].enabled then color = {155, 155, 155} end

                if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].new then
                    common.drawTexture(textures.newTexture, self.x - 6, y - 5, self.x + 36, y + 45)
                    common.drawText(self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].text, self.x + 30, y + 2, "SUBTITLES", 0.6, 1.8, "LEFT", self.w, color[1], color[2], color[3], 255, 0, 0)
                else
                    common.drawText(self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].text, self.x + 5, y + 2, "SUBTITLES", 0.6, 1.8, "LEFT", self.w, color[1], color[2], color[3], 255, 0, 0)
                end

                if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].enabled and self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].locked > 0 then
                    if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].locked == 1 then common.drawTexture(textures.lockTexture, self.x + self.w - 50, y - 5, self.x + self.w + 8, y + 45, unpack(color))
                    else common.drawTexture(textures.menuTextures["lockarena"], self.x + self.w - 50, y - 5, self.x + self.w + 8, y + 45, 203, 54, 148) end
                else
                    if rowType == 2 then
                        if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].checked then common.drawTexture(textures.menuTextures["tickbox"..prop], self.x + self.w - 50, y - 5, self.x + self.w + 8, y + 45)
                        else common.drawTexture(textures.menuTextures["shopbox"..prop], self.x + self.w - 50, y - 5, self.x + self.w + 8, y + 45) end
                    elseif rowType == 3 then
                        if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].crossed then common.drawTexture(textures.menuTextures["crossbox"..prop], self.x + self.w - 50, y - 5, self.x + self.w + 8, y + 45)
                        else common.drawTexture(textures.menuTextures["shopbox"..prop], self.x + self.w - 50, y - 5, self.x + self.w + 8, y + 45) end
                    elseif rowType == 4 then
                        common.drawTexture(textures.menuTextures["tick"], self.x + self.w - 50, y - 5, self.x + self.w + 8, y + 45, unpack(color))
                    elseif rowType == 5 then
                        common.drawText(self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].rtext, self.x + self.w - 10, y + 2, "SUBTITLES", 0.6, 1.8, "RIGHT", - self.w, color[1], color[2], color[3], 255, 0, 0)
                    elseif rowType == 6 then
                        common.mad.draw_text(tostring(self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].price).."$", common.convertX(self.x + self.w - 10), common.convertY(y + 2), common.mad.font_style.SUBTITLES, common.convertX(0.6), common.convertY(1.8), common.mad.font_align.RIGHT, -self.w, true, false, color[1], color[2], color[3], 255, 0, 0)
                    elseif rowType == 7 then
                        common.drawTexture(textures.menuTextures["chips"..prop], self.x + self.w - 30, y + 3, self.x + self.w, y + 40)
                        common.drawText(tostring(self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].price), self.x + self.w - 23, y + 2, "SUBTITLES", 0.6, 1.8, "RIGHT", - self.w, color[1], color[2], color[3], 255, 0, 0)
                    elseif rowType == 8 then
                        common.drawText(string.format(prop, self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].list[self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].listIndex]), self.x + self.w - 10, y + 2, "SUBTITLES", 0.6, 1.8, "RIGHT", - self.w, color[1], color[2], color[3], 255, 0, 0)
                    elseif rowType == 9 then
                        if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].medal == 1 then common.drawTexture(textures.menuTextures["medal"], self.x + self.w - 50, y - 5, self.x + self.w + 8, y + 45, 244, 183, 82)
                        elseif self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].medal == 2 then common.drawTexture(textures.menuTextures["medal"], self.x + self.w - 50, y - 5, self.x + self.w + 8, y + 45, 130, 130, 130)
                        elseif self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].medal == 3 then common.drawTexture(textures.menuTextures["medal"], self.x + self.w - 50, y - 5, self.x + self.w + 8, y + 45, 181, 130, 99)
                        end
                    elseif rowType == 10 then
                        if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].badge[2] then
                            common.drawTexture(textures.badges[self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(i)].badge[1]..prop], self.x + self.w - 47, y - 5, self.x + self.w + 3, y + 45)
                        end
                    end
                end
                
                y = y + 38
            end

            if self.structure["panel"..tostring(self.structure.currentPanel)].maxID > 10 then 
                common.drawTexture(textures.rect, self.x, y + 1, self.x + self.w, y + 39, 0, 0, 0)
                common.drawTexture(textures.updownarrowTexture, self.x + (self.w/2) - 23, y - 2, self.x + (self.w/2) + 23, y + 42)
                y = y + 39
            end

            if self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].description[1] ~= nil then
                common.drawTexture(textures.rect, self.x, y + 2, self.x + self.w, y + 7, 0, 0, 0)
                lenght = self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].description[2] - 1
                common.drawTexture(textures.bgdTexture, self.x, y + 2, self.x + self.w, y + 38 + (35 * lenght))
                common.drawText(self.structure["panel"..tostring(self.structure.currentPanel)]["row"..tostring(self.structure["panel"..tostring(self.structure.currentPanel)].currentID)].description[1], self.x + 5, y + 3, "SUBTITLES", 0.55, 1.75, "LEFT", self.w, 255, 255, 255, 255, 0, 0)
            end            
        end

        for event, key in pairs(self.keys) do
            if wasKeyPressed(key) then self:performEvent(event, key) end
        end
    end

    if isPlayerControllable(playerHandle) then
        if type(self.openKey) ~= "table" then 
            if wasKeyPressed(self.openKey) then 
                self:performEvent("open") 
            end
        else 

              if isKeyDown(self.openKey[1]) and wasKeyPressed(self.openKey[2]) then 
                self:performEvent("open") 
              end
    end
    end

end


return INT_MENU